/*
** ============================================================================
**
** FILE
**  radio.c
**
** DESCRIPTION
**  This file contains the source code for Raw Data Mode
**
** CREATED
**  Silicon Laboratories Ltd
**  Sunnyvale Office
**
** COPYRIGHT
**  Copyright 2009 Silicon Laboratories, Inc.  
**	http://www.silabs.com
**
** ============================================================================
*/

								/* ======================================================== *
								 *									INCLUDE					*
							 	 * ======================================================== */
#include "user_setup.h"

#if DEGLITCH_METHOD == 1

#include "C8051F930_defs.h"
#include "compiler_defs.h"
#include "deglitch_1.h"

								/* ======================================================== *
								 *							Function PROTOTYPES				* 
								 * ======================================================== */


								/* ======================================================== *
								 *							Global VARIABLES				* 
								 * ======================================================== */

bit Sample_2;
bit Sample_3;
bit Sample_4;
bit Sample_5;
bit Sample_6;
bit Sample_7;

bit ValidData_1;
bit ValidData_2;
bit LastValidData_1;
bit LastValidData_2;
bit DelayedSample_1;
bit DelayedSample_2;
bit DelayedSample_3;
bit DelayedSample_4;
bit DelayedSample_5;
bit databit;


#ifdef PACKET_HANDLER
extern U8 pointer;
extern U8 DataBitBuffer [2][32];
extern bit fBitBufferFull;
extern U8 fBufferSelect, fLastBufferSelect;
#endif

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void Deglitch1_Init(void)
  +
  + DESCRIPTION:    Initialize the deglitch 1 algorithm
  +
  +	INPUT:			None
  +
  + RETURN:         None
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void Deglitch1_Init(void)
{

	//Initialize the variables
	Sample_2 = 0;
	Sample_3 = 1;
	Sample_4 = 0;
	Sample_5 = 1;
	Sample_6 = 0;
	Sample_7 = 1;
	ValidData_1 = 1;
	LastValidData_1 = 0;
	ValidData_2 = 0;
	LastValidData_2 = 1;
	DelayedSample_1 = 0;
	DelayedSample_2 = 0;
	DelayedSample_3 = 0;
	DelayedSample_4 = 0;
	DelayedSample_5 = 0;
	databit = 0;


	// INT1 input: P0.7, active high, edge sensitive
	IT1 = 1;
//    IT01CF |= 0xF0;
    IT01CF |= ( 0x80 | ( RX_CLK_PIN << 4 ) );


}

void Deglitch1_Stop(void)
{
	// disable IT
	IE0 = 0;
    EX1 = 0;
}

void Deglitch1_Start()
{

	// enable IT
	EX1 = 1;

}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void Deglitch1(void)
  +
  + DESCRIPTION:    Algorithm for deglitching raw data
  +					
  +
  +	INPUT:			None
  +
  + RETURN:         None
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
INTERRUPT(INT1_ISR, INTERRUPT_INT1) 
{


		// Oversampled Bit Clock from the RF chip is connected to micro-controller
		// input. This clock is used to remove glitches on received data.
		// The DATA is the actual received data from the chip. It is sampled by the
		// oversampled clock. 
		// The filtering is done in 2 phases. Output of first filtering
		// phase is given to the next phase.
		// In each phase, 7samples of data are stored and based on logical
		// analysis, glitches that are only 1 sample wide are filtered.


		if ( RX_DATA == Sample_2 )
		{
			if ( RX_DATA == Sample_3 )
			{
				ValidData_1 = Sample_3;
			}
			else
			{
				if ( RX_DATA == Sample_4 )
				{
					if ( RX_DATA == Sample_5 )
					{
						ValidData_1 = Sample_5;
						Sample_3 = Sample_5;
					}
					else
					{
						ValidData_1 = Sample_4;
						Sample_3 = Sample_4;
					}
				}
				else
				{
					if ( RX_DATA == Sample_5 )
					{
						ValidData_1 = Sample_5;
						Sample_3 = Sample_5;
						Sample_4 = Sample_5;
					}
					else
					{
						ValidData_1 = Sample_5;
					}
				}
			}
		}
		else
		{
			if ( RX_DATA == Sample_3 )
			{
				if ( RX_DATA == Sample_4 )
				{
					ValidData_1 = Sample_4;
					Sample_2 = Sample_4;
				}
				else
				{
					ValidData_1 = Sample_5;
					Sample_3 = Sample_5;
				}
			}
			else
			{
				if ( RX_DATA == Sample_4 )
				{
					if ( RX_DATA == Sample_5 )
					{
						ValidData_1 = Sample_3;
					}
					else
					{
						ValidData_1 = Sample_5;
						Sample_4 = Sample_3;
					}
				}
				else
				{
					ValidData_1 = Sample_3;
				}
			}
		}
		if ( ValidData_1 == DelayedSample_1 )
		{
			if ( ValidData_1 == DelayedSample_2 )
			{
				ValidData_2 = DelayedSample_2;
			}
			else
			{
				if ( ValidData_1 == DelayedSample_3 )
				{
					if ( ValidData_1 == DelayedSample_4 )
					{
						ValidData_2 = DelayedSample_4;
						DelayedSample_2 = DelayedSample_4;
					}
					else
					{
						ValidData_2 = DelayedSample_3;
						DelayedSample_2 = DelayedSample_3;
					}
				}
				else
				{
					if ( ValidData_1 == DelayedSample_4 )
					{
						ValidData_2 = DelayedSample_4;
						DelayedSample_2 = DelayedSample_4;
						DelayedSample_3 = DelayedSample_4;
					}
					else
					{
						ValidData_2 = DelayedSample_4;
					}
				}
			}
		}
		else
		{
			if ( ValidData_1 == DelayedSample_2 )
			{
				if ( ValidData_1 == DelayedSample_3 )
				{
					ValidData_2 = DelayedSample_3;
					DelayedSample_1 = DelayedSample_3;
				}
				else
				{
					ValidData_2 = DelayedSample_4;
					DelayedSample_2 = DelayedSample_4;
				}
			}
			else
			{
				if ( ValidData_1 == DelayedSample_3 )
				{
					if ( ValidData_1 == DelayedSample_4 )
					{
						ValidData_1 = DelayedSample_2;
					}
					else
					{
						ValidData_2 = DelayedSample_4;
						DelayedSample_3 = DelayedSample_2;
					}
				}
				else
				{
					ValidData_2 = DelayedSample_2;
				}
			}
		}
		// Register the Data Samples
		Sample_7 = Sample_6;
		Sample_6 = Sample_5;
		Sample_5 = Sample_4;
		Sample_4 = Sample_3;
		Sample_3 = Sample_2;
		Sample_2 = RX_DATA;
		DelayedSample_4 = DelayedSample_3;
		DelayedSample_3 = DelayedSample_2;
		DelayedSample_2 = DelayedSample_1;
		DelayedSample_1 = ValidData_1;
		databit = ValidData_2;

		LED3 = databit;
		LED4 = ~databit;

#ifdef PACKET_HANDLER

		DataBitBuffer[fBufferSelect][pointer] = databit;
	
		pointer++;
		if ( pointer > 31 )
		{
			pointer = 0;
			fLastBufferSelect = fBufferSelect;
			if(fBufferSelect == 0)
			{
				fBufferSelect = 1;
			}
			else
			{
				fBufferSelect = 0;
			}
			fBitBufferFull = 1;
		}

#endif

}

#endif