#include "user_setup.h"

#if DEGLITCH_METHOD == 2

#include "C8051F930_defs.h"
#include "compiler_defs.h"
#include "deglitch_2.h"





U8 count;

#if WINDOWSIZE > 8
U16 shiftreg;
#else
U8 shiftreg;
#endif

#ifdef PACKET_HANDLER
extern U8 pointer;
extern U8 DataBitBuffer [2][32];
extern bit fBitBufferFull;
extern U8 fBufferSelect, fLastBufferSelect;
#endif


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME: Int0_Init() 
  +
  + DESCRIPTION:   INT0 interrupt setup routine
  + 					
  + INPUT:		   none
  +
  + RETURN:        none 
  +
  + NOTES:         none
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

void Deglitch2_Init()
{

	shiftreg = 0;
	count = 0;

	// int0 input: P0.7, active high, edge sensitive
	IT0 = 1;
//    IT01CF |= 0x0F;
	IT01CF |= ( 0x08 | RX_CLK_PIN );

}

void Deglitch2_Stop()
{

	// disable IT
	IE0 = 0;
	EX0 = 0;

}


void Deglitch2_Start()
{

	// enable IT
	EX0 = 1;

}


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME: INTERRUPT(INT0_ISR, INTERRUPT_INT0) 
  +
  + DESCRIPTION:   INT0 interrupt routine
  + 					
  + INPUT:		   none
  +
  + RETURN:        none 
  +
  + NOTES:         none
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
INTERRUPT(INT0_ISR, INTERRUPT_INT0) 
{

	bit databit;

	// RX_DATA: raw data input, P2.2
	// deglitched output: LED 3, P1.6	
	

	if ( shiftreg & BITMASK )
		count--;

	shiftreg <<= 1;

	if ( RX_DATA )
	{
		shiftreg++;
		count++;
	}

	shiftreg &= WINDOWMASK;


	if ( count > BITLIMIT )
		databit = 1;
	else
		databit = 0;
	
	LED3 = databit;
	LED4 = ~(databit);
	
#ifdef PACKET_HANDLER

	DataBitBuffer[fBufferSelect][pointer] = databit;
	

	pointer++;
	if ( pointer > 31 )
	{
		pointer = 0;
		fLastBufferSelect = fBufferSelect;
		if(fBufferSelect == 0)
		{
			fBufferSelect = 1;
		}
		else
		{
			fBufferSelect = 0;
		}
		fBitBufferFull = 1;
	}

#endif

}

#endif
