#include "spi.h"
#include "radio.h"
#include "user_setup.h"

extern U32 RX_CLK_Clock_Rate;


void Radio_Init ( void )
{
	U8 ItStatus1, ItStatus2;

	//read interrupt status registers to clear the interrupt flags and release NIRQ pin
	ItStatus1 = SpiReadRegister(0x03);
	ItStatus2 = SpiReadRegister(0x04);

	//SW reset   
	SpiWriteRegister(0x07, 0x80);

	//wait for chip ready interrupt from the radio (while the nIRQ pin is high)
	while ( NIRQ == 1);

	//read interrupt status registers to clear the interrupt flags and release NIRQ pin
	ItStatus1 = SpiReadRegister(0x03);
	ItStatus2 = SpiReadRegister(0x04);

	//set the physical parameters

	//set  Crystal Oscillator Load Capacitance register
	SpiWriteRegister(0x09, CRYSTAL_OSC_LOAD_CAP);

	//set the center frequency to 870.8 MHz
	SpiWriteRegister(0x75, FREQ_BAND_SELECT);														
	SpiWriteRegister(0x76, NOM_CARRIER_FREQ_1);														
	SpiWriteRegister(0x77, NOM_CARRIER_FREQ_0);

	// packet handler settings
	SpiWriteRegister(0x35, 0xF8);
	SpiWriteRegister(0x71, 0x63);

	// set to avoid false packet detection
	SpiWriteRegister(0x30, 0x84);
	SpiWriteRegister(0x32, 0x0F);
	SpiWriteRegister(0x33, 0x77);
	SpiWriteRegister(0x08, 0x10);

	// set GPIOs
	SpiWriteRegister(0x0b, 0xD4);				
	SpiWriteRegister(0x0c, 0xCF);
	SpiWriteRegister(0x51, 0x36);


/////////////////////////////////////////////////////////////////////////////
/////////////////////// FINAL MODEM SETTINGS ////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

// setting #1
#ifdef SETTING_1

	// Modem Parameters
	SpiWriteRegister(0x56, 0x40);
	SpiWriteRegister(0x1C, 0x3E);
	SpiWriteRegister(0x1D, 0x58);
	SpiWriteRegister(0x1E, 0x0A);
	SpiWriteRegister(0x20, 0x0D);
	SpiWriteRegister(0x21, 0x09);
	SpiWriteRegister(0x22, 0xD4);
	SpiWriteRegister(0x23, 0x95);
	SpiWriteRegister(0x24, 0x27);
	SpiWriteRegister(0x25, 0x54);
	SpiWriteRegister(0x2A, 0x11);
	SpiWriteRegister(0x71, 0x63);

	RX_CLK_Clock_Rate = 23914;

#endif


// setting #2
#ifdef SETTING_2

	// Modem Parameters
	SpiWriteRegister(0x56, 0x40);
	SpiWriteRegister(0x1C, 0xAE);
	SpiWriteRegister(0x1D, 0x68);
	SpiWriteRegister(0x1E, 0x0A);
	SpiWriteRegister(0x20, 0x3E);
	SpiWriteRegister(0x21, 0x02);
	SpiWriteRegister(0x22, 0x0C);
	SpiWriteRegister(0x23, 0x49);
	SpiWriteRegister(0x24, 0x04);
	SpiWriteRegister(0x25, 0x60);
	SpiWriteRegister(0x2A, 0xB0);
	SpiWriteRegister(0x71, 0x63);

	RX_CLK_Clock_Rate = 48500;

#endif

// setting #3
#ifdef SETTING_3

	// Modem Parameters
	SpiWriteRegister(0x56, 0x40);
	SpiWriteRegister(0x1C, 0x1E);
	SpiWriteRegister(0x1D, 0x58);
	SpiWriteRegister(0x1E, 0x1A);
	SpiWriteRegister(0x1F, 0x03);
	SpiWriteRegister(0x20, 0x68);
	SpiWriteRegister(0x21, 0x01);
	SpiWriteRegister(0x22, 0x3A);
	SpiWriteRegister(0x23, 0x92);
	SpiWriteRegister(0x24, 0x00);
	SpiWriteRegister(0x25, 0xCB);
	SpiWriteRegister(0x2A, 0x61);
	SpiWriteRegister(0x71, 0x63);

	RX_CLK_Clock_Rate = 19360;

#endif

// setting #4
#ifdef SETTING_4

	// Modem Parameters
	SpiWriteRegister(0x56, 0x40);
	SpiWriteRegister(0x1C, 0xAE);
	SpiWriteRegister(0x1D, 0x58);
	SpiWriteRegister(0x1E, 0x1A);
	SpiWriteRegister(0x20, 0x3E);
	SpiWriteRegister(0x21, 0x02);
	SpiWriteRegister(0x22, 0x0C);
	SpiWriteRegister(0x23, 0x49);
	SpiWriteRegister(0x24, 0x04);
	SpiWriteRegister(0x25, 0x60);
	SpiWriteRegister(0x2A, 0x61);
	SpiWriteRegister(0x71, 0x63);

	RX_CLK_Clock_Rate = 48500;

#endif

// setting #5
#ifdef SETTING_5

	// Modem Parameters
	SpiWriteRegister(0x56, 0x40);
	SpiWriteRegister(0x1C, 0x8E);
	SpiWriteRegister(0x1D, 0x68);
	SpiWriteRegister(0x1E, 0x0A);
	SpiWriteRegister(0x20, 0xFA);
	SpiWriteRegister(0x21, 0x00);
	SpiWriteRegister(0x22, 0x83);
	SpiWriteRegister(0x23, 0x12);
	SpiWriteRegister(0x24, 0x00);
	SpiWriteRegister(0x25, 0x55);
	SpiWriteRegister(0x2A, 0xB0);
	SpiWriteRegister(0x71, 0x63);

	RX_CLK_Clock_Rate = 48000;

#endif



// setting #6
#ifdef SETTING_6

	// Modem Parameters
	SpiWriteRegister(0x1F, 0x00);
	SpiWriteRegister(0x1C, 0x4E);
	SpiWriteRegister(0x20, 0x7D);
	SpiWriteRegister(0x21, 0xE3);
	SpiWriteRegister(0x22, 0x86);
	SpiWriteRegister(0x23, 0x25);
	SpiWriteRegister(0x24, 0x01);
	SpiWriteRegister(0x25, 0x08);
	SpiWriteRegister(0x2C, 0x18);
	SpiWriteRegister(0x2D, 0xD0);
	SpiWriteRegister(0x2E, 0x2E);
	SpiWriteRegister(0x71, 0x69);
    SpiWriteRegister(0x70, 0x0E);

	RX_CLK_Clock_Rate = 13600;

#endif

// setting #7
#ifdef SETTING_7

	// Modem Parameters
	SpiWriteRegister(0x1F, 0x00);
	SpiWriteRegister(0x1C, 0xC7);
	SpiWriteRegister(0x20, 0x7D);
	SpiWriteRegister(0x21, 0x63);
	SpiWriteRegister(0x22, 0x86);
	SpiWriteRegister(0x23, 0x25);
	SpiWriteRegister(0x24, 0x01);
	SpiWriteRegister(0x25, 0x08);
	SpiWriteRegister(0x2C, 0x18);
	SpiWriteRegister(0x2D, 0xD0);
	SpiWriteRegister(0x2E, 0x2E);
	SpiWriteRegister(0x71, 0x69);
    SpiWriteRegister(0x70, 0x0E);

	RX_CLK_Clock_Rate = 41300;

#endif

// setting #8
#ifdef SETTING_8

	SpiWriteRegister(0x1F, 0x00);
	SpiWriteRegister(0x1C, 0x2E);
	SpiWriteRegister(0x20, 0x7D);
	SpiWriteRegister(0x21, 0x63);
	SpiWriteRegister(0x22, 0x86);
	SpiWriteRegister(0x23, 0x25);
	SpiWriteRegister(0x24, 0x01);
	SpiWriteRegister(0x25, 0x08);
	SpiWriteRegister(0x2C, 0x18);
	SpiWriteRegister(0x2D, 0xD0);
	SpiWriteRegister(0x2E, 0x2E);
	SpiWriteRegister(0x71, 0x69);
    SpiWriteRegister(0x70, 0x0E);

	RX_CLK_Clock_Rate = 55100;

#endif


/////////////////////////////////////////////////////////////////////////////
/////////////////////// END MODEM SETTINGS //////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

	// Set Non default registers
	SpiWriteRegister(0x69, 0x60);

}


void StopReceiver(void)
{
	// Turn Receiver OFF
	SpiWriteRegister(0x07, 0x01);
}


void StartReceiver(void)
{
	// Turn Receiver ON
	SpiWriteRegister(0x07, 0x04);
}