/*
** ============================================================================
**
** FILE
**  retime_phandler.c
**
** DESCRIPTION
**  Rrealize the retime and packethandler functions
**
** CREATED
**  Silicon Laboratories Hungary Ltd
**
** COPYRIGHT
**  Copyright 2010 Silicon Laboratories, Inc.  
**	http://www.silabs.com
**
** ===========================================================================*/

#include "user_setup.h"

#ifdef PACKET_HANDLER

#include "retime_phandler.h"




                /* ======================================= *
                 *  		G L O B A L  V A R I A B L E  	  *
                 * ======================================= */

//these variables should be cleared once before call receive function
bit fRetiming;									
bit fPacketHandler;							
U8 SyncWordBitCnt;								
U8 PayloadBitCnt;									
bit fSyncWordFound;								
bit fPacketReceived; 							
U8 PayloadBuffer[PAYLOAD_BYTE_LENGTH];
U8 ComparePayload[PAYLOAD_BYTE_LENGTH] = PAYLOAD_CONTENT;


//this variable should be set once before call receive function
U8 SyncWord;	
//this variable should be set to logical one once before call receive function
U16 DeglitchedBitCnt;		
U8 BitValue;

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:	void BitsRetiming(U8 * InputBuffer, U8 NmbrOfInputBits, U16 SamplePerBit, U8 * OutputBuffer, U8 * NmbrOfOutputBits)
  +
  + DESCRIPTION:    	Retime the deglitched bits
  +
  + INPUT:			 	InputBuffer - input buffer, the deglitched bits
  +						NmbrOfInputBits - the number of the deglitched bit
  +						SamplePerBit	- sample per bits
  +						OutputBuffer - output buffer, the retimed bits
  +						NmbrOfOutputBits - the number of retimed bits	
  +
  + RETURN:       
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/ 


void BitsRetiming(U8 *InputBuffer, U8 NmbrOfInputBits, U16 SamplePerBit, U8 *OutputBuffer, U8 *NmbrOfOutputBits)
{

	
	U8 i,j;	
	U8 temp1, temp2;
	U16 DBitLimit;

	i = 0;		
	DBitLimit = ( (U16)SamplePerBit << 3 );
	*NmbrOfOutputBits = 0;

	while ( i < NmbrOfInputBits )								//go through all of the input bits 				
	{
		if ( ( *( InputBuffer + i ) == BitValue ) && ( DeglitchedBitCnt < DBitLimit ) ) 		//if the next bit is the same
		{
			DeglitchedBitCnt++;									//increase the deglitched bit counter 
		}

		else													//if the next bit isn't the same
		{

			DeglitchedBitCnt <<= 2;
			temp1 = (U8)( DeglitchedBitCnt / SamplePerBit );	//estimate the number of bits
			temp2 = (U8)( DeglitchedBitCnt % SamplePerBit );  
			
			for ( j = 0; j < temp1; j++ )
			{

				*OutputBuffer++ = BitValue; 		//save the retimed bits
				*NmbrOfOutputBits += 1; 
			}
			
			if ( temp2 > ( SamplePerBit >> 1 ) )
			{

				*OutputBuffer++ = BitValue;			//save the retimed bit
				*NmbrOfOutputBits += 1;
			}

			BitValue = *( InputBuffer + i );		//save the new bit value
			DeglitchedBitCnt = 1;

		}

		i++;

	}

}


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:	void PacketHandler(U8 * InputBuffer, U8 NmbrOfInputBits)
  +
  + DESCRIPTION:    	This function process the retimed data, finds the sync word and gives back the payload
  +
  + INPUT:			 	InputBuffer - the retimed bits 
  +						NmbrOfInputBits - the retimed bit number
  +
  + RETURN:       
  +
  + NOTES:          the process starts always with the MSB bit
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/ 


void PacketHandler(U8 * InputBuffer, U8 NmbrOfInputBits)
{
	U8 i;
	static U8 TempByte,BitCnt, ByteCnt;
	static U32 SyncWordTempByte;

	i = 0;

	while(i< NmbrOfInputBits)
	{


		//check for synch word
		if(fSyncWordFound == 0)				 		//if the start of the sync word hasn't found yet
		{

		
			//save bits into bytes
			SyncWordTempByte <<= 1;
			if(*(InputBuffer+i) == 1)
			{
				SyncWordTempByte |= 0x00000001;
			}
			else
			{
				SyncWordTempByte &= SYNCWORD_MASK;
			}
			
			if(SyncWordTempByte == SYNCWORD)
			{//synchron word found

				fSyncWordFound = 1;
				BitCnt = 0;
				ByteCnt = 0;
				LED1 = 1;

			}
		}
		else
		{

			//save bits into bytes
			TempByte <<= 1;
			if(*(InputBuffer+i) == 1)
			{
				TempByte |= 0x01;
			}
			else
			{
				TempByte &= 0xFE;
			}

			if( ++BitCnt == 8 )
			{//save the next byte into the buffer
				PayloadBuffer[ByteCnt++] = TempByte;
				BitCnt = 0;
				//check whether the number of specified bytes already received or not
				if(ByteCnt == PAYLOAD_BYTE_LENGTH)
				{//packet is received

					fPacketReceived = 1;
					fSyncWordFound = 0;

				}				
			}
		}


		i++;
	}
}

#endif
