#ifndef USER_SETUP_H
#define USER_SETUP_H

#include "C8051F930_defs.h"

// Crystal Oscillator Load Capacitance register: set the appropriate value to tune the 30MHz crystal
#define CRYSTAL_OSC_LOAD_CAP	0xD5	// address: 0x09

// Carrier frequency setting registers: set the desires receiving frequency here
// Frequency Band Select register
#define FREQ_BAND_SELECT		0x73	// address: 0x75
// Nominal Carrier Frequency 1 register
#define NOM_CARRIER_FREQ_1		0x86	// address: 0x76
// Nominal Carrier Frequency 0 register
#define NOM_CARRIER_FREQ_0		0xF7	// address: 0x77

// uncomment the appropriate modem setting; see the appnote for details
#define SETTING_1
//#define SETTING_2
//#define SETTING_3
//#define SETTING_4
//#define SETTING_5
//#define SETTING_6
//#define SETTING_7
//#define SETTING_8

// define here the deglitch method to be used: 1 or 2
#define DEGLITCH_METHOD			1

// if deglitch method #2 was chosen then define the size of the glitch filtering window
#define WINDOWSIZE				15					// size of the glitch filtering window

// if you do not want to use the packet handler then comment out the next line
#define PACKET_HANDLER


// if the packet handler is enabled specify here the details of the transmitted data

// datarate
#define DATARATE				(1000)				// datarate of the received data

// synchronization word properties
#define SYNCWORD				0xAA2DD4			//sync word pattern max 4 bytes
#define SYNCWORD_BIT_LENGTH		24					//length of the sync word in bits, max 32

// payload properties
#define PAYLOAD_CONTENT			{ 0x01, 0x23, 0x45, 0x67, 0x89 }	// the content of the payload
#define PAYLOAD_BYTE_LENGTH		5									// length of the payload in bytes	


/* ================================================================ *
 * C8051F930 Pin definitions for Software Development Board         *
 * 					(using compiler_def.h macros) 					*
 *	=============================================================== */

// the RX_CLK signal must be connected to P0 port because the INT0/INT1 interrupts can be assigned only to this port
#define	RX_CLK_PIN			7
SBIT	(RX_CLK,	SFR_P0, RX_CLK_PIN);

SBIT	(RX_DATA,	SFR_P2, 2);

SBIT	(NSS,		SFR_P1, 3);
SBIT	(NIRQ,		SFR_P0, 6);
SBIT	(PB1,		SFR_P0, 0);
SBIT	(PB2,		SFR_P0, 1);
SBIT	(PB3,		SFR_P2, 0);
SBIT	(PB4,		SFR_P2, 1);
SBIT	(LED1,		SFR_P1, 4);
SBIT	(LED2,		SFR_P1, 5);
SBIT	(LED3,		SFR_P1, 6);
SBIT	(LED4,		SFR_P1, 7);

// calculated parameters
// do not change
#define SYNCWORD_MASK			( ( (1L) << SYNCWORD_BIT_LENGTH ) - 2 )
#define WINDOWMASK				( ( 1  << WINDOWSIZE ) - 1 )
#define BITLIMIT				( WINDOWSIZE / 2 )
#define BITMASK					( ( 1  << ( WINDOWSIZE - 1 ) ) )



#endif